@@ -15,7 +15,10 @@
 # 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 
+import copy
 import pikepdf
+import traceback
+import sys
 from . import metadata
 
 from decimal import Decimal
@@ -39,26 +42,44 @@ def _mediabox(row, angle, angle0, box):
         x2_new = x2 - (x2 - x1) * crop[1]
         y1_new = y1 + (y2 - y1) * crop[3]
         y2_new = y2 - (y2 - y1) * crop[2]
-        # TODO: check if Decimal is still needed now that we dropped PyPDF2
+        # pikepdf converts float to Decimal in most cases
         return [Decimal(v) for v in [x1_new, y1_new, x2_new, y2_new]]
 
 
+_report_pikepdf_err = True
+
+
 def export(input_files, pages, file_out, mdata):
+    global _report_pikepdf_err
     pdf_output = pikepdf.Pdf.new()
     pdf_input = [pikepdf.open(p.copyname) for p in input_files]
     for row in pages:
         current_page = pdf_input[row[2] - 1].pages[row[3] - 1]
         angle = row[6]
         angle0 = current_page.Rotate if '/Rotate' in current_page else 0
+        new_page = pdf_output.copy_foreign(current_page)
+        # Workaround for pikepdf <= 1.10.1
+        # https://github.com/pikepdf/pikepdf/issues/80#issuecomment-590533474
+        try:
+            new_page = copy.copy(new_page)
+        except TypeError:
+            if _report_pikepdf_err:
+                _report_pikepdf_err = False
+                traceback.print_exc()
+                print("Current pikepdf version {}, required pikepdf version "
+                      "1.7.0 or greater. Continuing but pdfarranger will not "
+                      "work properly.".format(pikepdf.__version__),
+                      file=sys.stderr)
         if angle != 0:
-            current_page.Rotate = angle + angle0
+            new_page.Rotate = angle + angle0
         cropped = _mediabox(row, angle, angle0, current_page.MediaBox)
         if cropped:
-            current_page.MediaBox = cropped
-        pdf_output.pages.append(pdf_output.copy_foreign(current_page))
+            new_page.MediaBox = cropped
+        pdf_output.pages.append(new_page)
     ppae = metadata.PRODUCER not in mdata
     with pdf_output.open_metadata(set_pikepdf_as_editor=ppae) as outmeta:
-        outmeta.load_from_docinfo(pdf_input[0].docinfo)
+        if len(pdf_input) > 0:
+            outmeta.load_from_docinfo(pdf_input[0].docinfo)
         for k, v in mdata.items():
             outmeta[k] = v
     pdf_output.save(file_out)
