@@ -48,6 +48,7 @@ if not os.path.exists(localedir):
 
 locale.setlocale(locale.LC_ALL, '')
 DOMAIN = 'pdfarranger'
+ICON_ID = 'com.github.jeromerobert.' + DOMAIN
 if os.name == 'nt':
     from ctypes import cdll
 
@@ -63,7 +64,7 @@ else:
         pass
 
 APPNAME = 'PDF Arranger'
-VERSION = '1.4.1'
+VERSION = '1.5.1'
 WEBSITE = 'https://github.com/jeromerobert/pdfarranger'
 LICENSE = 'GNU General Public License (GPL) Version 3.'
 
@@ -74,13 +75,13 @@ gi.check_version('3.10.2')
 gi.require_version('Gtk', '3.0')
 from gi.repository import Gtk
 
-if Gtk.check_version(3, 4, 0):
+if Gtk.check_version(3, 12, 0):
     raise Exception('You do not have the required version of GTK+ installed. ' +
                     'Installed GTK+ version is ' +
                     '.'.join([str(Gtk.get_major_version()),
                               str(Gtk.get_minor_version()),
                               str(Gtk.get_micro_version())]) +
-                    '. Required GTK+ version is 3.4 or higher.')
+                    '. Required GTK+ version is 3.12 or higher.')
 
 from gi.repository import Gdk
 from gi.repository import GObject  # for using custom signals
@@ -159,14 +160,14 @@ class Config(object):
     def maximized(self):
         return self.data.getboolean('window', 'maximized', fallback=False)
 
-    def set_maximized(self, m):
-        self.data.set('window', 'maximized', str(m))
+    def set_maximized(self, maximized):
+        self.data.set('window', 'maximized', str(maximized))
 
     def zoom_level(self):
         return self.data.getint('DEFAULT', 'zoom-level', fallback=0)
 
-    def set_zoom_level(self, l):
-        self.data.set('DEFAULT', 'zoom-level', str(l))
+    def set_zoom_level(self, level):
+        self.data.set('DEFAULT', 'zoom-level', str(level))
 
     def save(self):
         conffile = Config._config_file()
@@ -203,16 +204,11 @@ def warn_dialog(func):
 
 def get_file_path_from_dnd_dropped_uri(uri):
     """Extracts the path from an uri"""
+    uri = uri[5:]  # remove 'file:'
     path = url2pathname(uri)  # escape special chars
-    path = path.strip('\r\n\x00')  # remove \r\n and NULL
-
-    # get the path to file
-    if path.startswith('file:\\\\\\'):  # windows
-        path = path[8:]  # 8 is len('file:///')
-    elif path.startswith('file://'):  # nautilus, rox
-        path = path[7:]  # 7 is len('file://')
-    elif path.startswith('file:'):  # xffm
-        path = path[5:]  # 5 is len('file:')
+    path = path.strip('\r\n\x00\x2F')  # remove \r\n and NULL and \
+    if os.name == 'posix':
+        path = '/' + path
     return path
 
 
@@ -252,11 +248,12 @@ class PdfArranger(Gtk.Application):
         self.iconview = None
         self.cellthmb = None
         self.progress_bar = None
-        self.progress_bar_timeout_id = 0
+        self.progress_bar_timeout_id = None
         self.popup = None
         self.is_unsaved = False
         self.zoom_level = None
         self.zoom_scale = None
+        self.target_is_intern = True
 
         self.export_directory = os.path.expanduser('~')
         self.import_directory = self.export_directory
@@ -269,6 +266,12 @@ class PdfArranger(Gtk.Application):
         self.rendering_thread = None
         self.export_file = None
 
+        # Clipboard for cut copy paste
+        if os.name == 'nt':
+            self.clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        else:
+            self.clipboard = Gtk.Clipboard.get(Gdk.Atom.intern('_SELECTION_PDFARRANGER', False))
+
     def do_open(self, files, _n, _hints):
         """ https://lazka.github.io/pgi-docs/Gio-2.0/classes/Application.html#Gio.Application.do_open """
         self.activate()
@@ -276,17 +279,42 @@ class PdfArranger(Gtk.Application):
         a = PageAdder(self)
         for f in files:
             a.addpages(f.get_path())
-        a.commit()
+        a.commit(select_added=False, add_to_undomanager=True)
         if len(files) == 1:
             self.set_unsaved(False)
 
+    def __build_from_file(self, path):
+        """ Return the path of a resource file """
+        # TODO: May be we could use Application.set_resource_base_path and
+        # get_menu_by_id in place of that
+        # Trying different possible locations
+        f = os.path.join(basedir, 'share', DOMAIN, path)
+        if not os.path.exists(f):
+            f = os.path.join(basedir, 'data', path)
+        if not os.path.exists(f):
+            f = '/usr/share/{}/{}'.format(DOMAIN, path)
+        if not os.path.exists(f):
+            f = '/usr/local/share/{}/{}'.format(DOMAIN, path)
+        b = Gtk.Builder()
+        b.set_translation_domain(DOMAIN)
+        b.add_from_file(f)
+        b.connect_signals(self)
+        return b
+
+    def __create_menus(self):
+        b = self.__build_from_file("menu.ui")
+        self.popup = Gtk.Menu.new_from_model(b.get_object("popup_menu"))
+        self.popup.attach_to_widget(self.window, None)
+        main_menu = self.uiXML.get_object("main_menu_button")
+        main_menu.set_menu_model(b.get_object("main_menu"))
+
     def __create_actions(self):
         # Both Gtk.ApplicationWindow and Gtk.Application are Gio.ActionMap. Some action are window
         # related some other are application related. As pdfarrager is a single window app does not
         # matter that much.
         self.window.add_action_entries([
             ('rotate', self.rotate_page_action, 'i'),
-            ('delete', self.clear_selected),
+            ('delete', self.on_action_delete),
             ('duplicate', self.duplicate),
             ('crop', self.crop_page_dialog),
             ('export-selection', self.choose_export_selection_pdf_name),
@@ -300,7 +328,15 @@ class PdfArranger(Gtk.Application):
             ('redo', self.undomanager.redo),
             ('split', self.split_pages),
             ('metadata', self.edit_metadata),
+            ('cut', self.on_action_cut),
+            ('copy', self.on_action_copy),
+            ('paste', self.on_action_paste, 'i'),
+            ('about', self.about_dialog),
         ])
+
+        main_menu = self.uiXML.get_object("main_menu_button")
+        self.window.add_action(Gio.PropertyAction.new("main-menu", main_menu, "active"))
+
         accels = [
             ('delete', 'Delete'),
             ('crop', 'c'),
@@ -311,23 +347,20 @@ class PdfArranger(Gtk.Application):
             ('export-selection', '<Ctrl>e'),
             ('quit', '<Ctrl>q'),
             ('import', 'Insert'),
-            ('zoom(5)', 'plus'),
-            ('zoom(-5)', 'minus'),
+            ('zoom(5)', ['plus', 'KP_Add']),
+            ('zoom(-5)', ['minus', 'KP_Subtract']),
             ('undo', '<Ctrl>z'),
             ('redo', '<Ctrl>y'),
+            ('cut', '<Ctrl>x'),
+            ('copy', '<Ctrl>c'),
+            ('paste(0)', '<Ctrl>v'),
+            ('paste(1)', '<Ctrl><Shift>v'),
+            ('main-menu', 'F10'),
         ]
         for a, k in accels:
-            self.set_accels_for_action("win." + a, [k])
+            self.set_accels_for_action("win." + a, [k] if isinstance(k, str) else k)
         # Disable actions
         self.iv_selection_changed_event()
-        # Display accelerators in the menus
-        for o in self.uiXML.get_objects():
-            if isinstance(o, Gtk.MenuItem) and o.get_action_name() is not None:
-                an = Gio.Action.print_detailed_name(o.get_action_name(),
-                                                    o.get_action_target_value())
-                a = self.get_accels_for_action(an)
-                if len(a) > 0:
-                    o.get_child().set_accel(*Gtk.accelerator_parse(a[0]))
         self.undomanager.set_actions(self.window.lookup_action('undo'),
                                      self.window.lookup_action('redo'))
 
@@ -339,22 +372,8 @@ class PdfArranger(Gtk.Application):
         if not os.path.exists(iconsdir):
             iconsdir = os.path.join(sharedir, 'data', 'icons')
         Gtk.IconTheme.get_default().append_search_path(iconsdir)
-        Gtk.Window.set_default_icon_name(DOMAIN)
-
-        # Import the user interface file, trying different possible locations
-        ui_path = os.path.join(basedir, 'share', DOMAIN, DOMAIN + '.ui')
-        if not os.path.exists(ui_path):
-            ui_path = os.path.join(basedir, 'data', DOMAIN + '.ui')
-        if not os.path.exists(ui_path):
-            ui_path = '/usr/share/{}/{}.ui'.format(DOMAIN, DOMAIN)
-        if not os.path.exists(ui_path):
-            ui_path = '/usr/local/share/{}/{}.ui'.format(DOMAIN, DOMAIN)
-
-        self.uiXML = Gtk.Builder()
-        self.uiXML.set_translation_domain(DOMAIN)
-        self.uiXML.add_from_file(ui_path)
-        self.uiXML.connect_signals(self)
-
+        Gtk.Window.set_default_icon_name(ICON_ID)
+        self.uiXML = self.__build_from_file(DOMAIN + '.ui')
         # Create the main window, and attach delete_event signal to terminating
         # the application
         self.window = self.uiXML.get_object('main_window')
@@ -406,6 +425,8 @@ class PdfArranger(Gtk.Application):
         self.iconview.set_item_width(-1)
 
         self.cellthmb = CellRendererImage()
+        self.cellthmb.set_padding(3, 3)
+        self.cellthmb.set_alignment(0.5, 0.5)
         self.iconview.pack_start(self.cellthmb, False)
         self.iconview.set_cell_data_func(self.cellthmb, self.set_cellrenderer_data, None)
         self.iconview.set_text_column(0)
@@ -433,6 +454,9 @@ class PdfArranger(Gtk.Application):
 
         self.sw.add_with_viewport(self.iconview)
 
+        self.model.connect('row-inserted', self.__update_num_pages)
+        self.model.connect('row-deleted', self.__update_num_pages)
+
         # Progress bar
         self.progress_bar = self.uiXML.get_object('progressbar')
 
@@ -455,8 +479,22 @@ class PdfArranger(Gtk.Application):
         self.iconview.override_background_color(Gtk.StateFlags.PRELIGHT,
                                                 color_prelight)
 
-        self.popup = self.uiXML.get_object('popup_menu')
-        self.popup.attach_to_widget(self.window, None)
+        # Set outline properties for iconview items i.e. cursor look
+        style_provider = Gtk.CssProvider()
+        css_data = """
+        iconview {
+            outline-color: alpha(currentColor, 0.8);
+            outline-style: dashed;
+            outline-offset: -2px;
+            outline-width: 2px;
+            -gtk-outline-radius: 2px;
+        }
+        """
+        style_provider.load_from_data(bytes(css_data.encode()))
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(),
+            style_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
         GObject.type_register(PDFRenderer)
         GObject.signal_new('update_thumbnail', PDFRenderer,
@@ -465,6 +503,7 @@ class PdfArranger(Gtk.Application):
                             GObject.TYPE_FLOAT])
         self.set_unsaved(False)
         self.__create_actions()
+        self.__create_menus()
 
     @staticmethod
     def set_cellrenderer_data(_column, cell, model, it, _data=None):
@@ -487,7 +526,7 @@ class PdfArranger(Gtk.Application):
         self.rendering_thread.connect('update_thumbnail', self.update_thumbnail)
         self.rendering_thread.start()
 
-        if self.progress_bar_timeout_id:
+        if self.progress_bar_timeout_id is not None:
             GObject.source_remove(self.progress_bar_timeout_id)
         self.progress_bar_timeout_id = \
             GObject.timeout_add(50, self.progress_bar_timeout)
@@ -525,6 +564,7 @@ class PdfArranger(Gtk.Application):
                                    % {'i1': cnt_finished, 'i2': cnt_all})
         if fraction >= 0.999:
             self.progress_bar.hide()
+            self.progress_bar_timeout_id = None
             return False
         elif not self.progress_bar.get_visible():
             self.progress_bar.show()
@@ -544,13 +584,13 @@ class PdfArranger(Gtk.Application):
             # scale*page_width*(1-crop_left-crop_right)
             item_width = int(max(row[4] * row[11] * (1. - row[7] - row[8])
                                  for row in self.model))
-            # FIXME: those are magic number found with my current GTK
-            #  style. This might not be portable.
-            min_col_spacing = 19
+            item_padding = self.iconview.get_item_padding()
+            cellthmb_xpad, _cellthmb_ypad = self.cellthmb.get_padding()
+            min_col_spacing = 7 + 2 * (item_padding + cellthmb_xpad)
             min_margin = 14
             iw_width = window.get_size()[0]
             # 2 * min_margin + col_num * item_width
-            #  + min_col_spacing * (col_num-1) = iw_width
+            #  + min_col_spacing * (col_num+1) = iw_width
             # min_margin+margin = min_col_spacing+col_spacing
             #  = (iw_width - col_num * item_width) / (col_num+1)
             col_num = (iw_width - 2 * min_margin - min_col_spacing) //\
@@ -716,17 +756,11 @@ class PdfArranger(Gtk.Application):
         if response == Gtk.ResponseType.ACCEPT:
             adder = PageAdder(self)
             for filename in chooser.get_filenames():
-                try:
-                    adder.addpages(filename)
-                except Exception as e:
-                    traceback.print_exc()
-                    chooser.destroy()
-                    self.error_message_dialog(e)
-                    return
-            adder.commit()
+                adder.addpages(filename)
+            adder.commit(select_added=False, add_to_undomanager=True)
         chooser.destroy()
 
-    def clear_selected(self, _action, _parameter, _unknown):
+    def clear_selected(self):
         """Removes the selected elements in the IconView"""
 
         self.undomanager.commit("Delete")
@@ -745,37 +779,172 @@ class PdfArranger(Gtk.Application):
                 self.iconview.select_path(path)
         self.iconview.grab_focus()
 
+    def copy_pages(self):
+        """Collect data from selected pages"""
+
+        model = self.iconview.get_model()
+        selection = self.iconview.get_selected_items()
+        selection.sort(key=lambda x: x.get_indices()[0])
+
+        data = []
+        for path in selection:
+            it = model.get_iter(path)
+            nfile, npage, angle = model.get(it, 2, 3, 6)
+            crop = model.get(it, 7, 8, 9, 10)
+            pdfdoc = self.pdfqueue[nfile - 1]
+            data.append('\n'.join([pdfdoc.filename,
+                                   str(npage),
+                                   str(angle)] +
+                                  [str(side) for side in crop]))
+        if data:
+            data = '\n;\n'.join(data)
+
+        return data
+
+    @staticmethod
+    def data_to_pageadder(data, pageadder):
+        """Data to pageadder"""
+
+        tmp = data.pop(0).split('\n')
+        filename = tmp[0]
+        npage, angle = [int(k) for k in tmp[1:3]]
+        crop = [float(side) for side in tmp[3:7]]
+        pageadder.addpages(filename, npage, angle, crop)
+
+    def paste_pages(self, data, before, ref_to, select_added):
+        """Paste pages to iconview"""
+
+        pageadder = PageAdder(self)
+        if ref_to:
+            pageadder.move(ref_to, before)
+
+        if not before:
+            data.reverse()
+        while data:
+            try:  # Clipboard can contain not expected data
+                self.data_to_pageadder(data, pageadder)
+            except (ValueError, FileNotFoundError):
+                return False
+        return pageadder.commit(select_added, add_to_undomanager=True)
+
+    def paste_pages_interleave(self, data, before, ref_to):
+        """Paste pages interleved to iconview"""
+
+        pageadder = PageAdder(self)
+        model = self.iconview.get_model()
+        iter_to = None
+
+        self.undomanager.commit("Paste")
+        self.set_unsaved(True)
+
+        while data:
+            try:  # Clipboard can contain not expected data
+                self.data_to_pageadder(data, pageadder)
+            except (ValueError, FileNotFoundError):
+                return
+
+            pageadder.move(ref_to, before)
+            pageadder.commit(select_added=False, add_to_undomanager=False)
+
+            if ref_to:
+                path = ref_to.get_path()
+                iter_to = model.get_iter(path)
+                iter_to = model.iter_next(iter_to)
+                if not before:
+                    iter_to = model.iter_next(iter_to)
+            if iter_to:
+                path = model.get_path(iter_to)
+                ref_to = Gtk.TreeRowReference.new(model, path)
+            else:
+                ref_to = None
+
+    def on_action_delete(self, _action, _parameter, _unknown):
+        """Removes the selected elements in the IconView"""
+
+        self.clear_selected()
+
+    def on_action_cut(self, _action, _param, _unknown):
+        """Cut selected pages to clipboard"""
+
+        data = self.copy_pages()
+        self.clipboard.set_text(data, -1)
+
+        self.clear_selected()
+
+    def on_action_copy(self, _action, _param, _unknown):
+        """Copy selected pages to clipboard"""
+
+        data = self.copy_pages()
+        self.clipboard.set_text(data, -1)
+
+    def on_action_paste(self, _action, mode, _unknown):
+        """Paste pages from clipboard"""
+
+        model = self.iconview.get_model()
+
+        selection = self.iconview.get_selected_items()
+        selection.sort(key=lambda x: x.get_indices()[0])
+
+        # mode = 0 paste after
+        # mode = 1 paste before
+        # mode = 2 paste interleave odd
+        # mode = 3 paste interveave even
+        mode = mode.get_int32()
+        if len(model) == 0:
+            before = True
+            ref_to = None
+        elif mode == 0:
+            before = False
+            if len(selection) == 0:
+                ref_to = None
+            else:
+                ref_to = Gtk.TreeRowReference.new(model, selection[-1])
+        else:
+            if mode == 3:
+                before = False
+            else:
+                before = True
+            if len(selection) == 0:
+                ref_to = Gtk.TreeRowReference.new(model, Gtk.TreePath(0))
+            else:
+                ref_to = Gtk.TreeRowReference.new(model, selection[0])
+
+        data = self.clipboard.wait_for_text()
+        if data:
+            data = data.split('\n;\n')
+            if mode == 0 or mode == 1:
+                self.paste_pages(data, before, ref_to, select_added=False)
+            elif mode == 2 or mode == 3:
+                self.paste_pages_interleave(data, before, ref_to)
+
     @staticmethod
     def iv_drag_begin(iconview, context):
-        """Sets custom icon on drag begin for multiple items selected"""
-        if len(iconview.get_selected_items()) > 1:
-            iconview.stop_emission('drag_begin')
-            Gtk.drag_set_icon_name(context, "gtk-dnd-multiple", 0, 0)
+        """Sets custom drag icon."""
+        selected_count = len(iconview.get_selected_items())
+        stock_icon = "gtk-dnd-multiple" if selected_count > 1 else "gtk-dnd"
+        iconview.stop_emission('drag_begin')
+        Gtk.drag_set_icon_name(context, stock_icon, 0, 0)
 
-    def iv_dnd_get_data(self, iconview, _context,
+    def iv_dnd_get_data(self, _iconview, _context,
                         selection_data, _target_id, _etime):
         """Handles requests for data by drag and drop in iconview"""
 
-        model = iconview.get_model()
-        selection = self.iconview.get_selected_items()
-        selection.sort(key=lambda x: x.get_indices()[0])
-        data = []
-        for path in selection:
-            target = str(selection_data.get_target())
-            if target == 'MODEL_ROW_INTERN':
+        target = str(selection_data.get_target())
+        if target == 'MODEL_ROW_INTERN':
+            self.target_is_intern = True
+            selection = self.iconview.get_selected_items()
+            selection.sort(key=lambda x: x.get_indices()[0])
+            data = []
+            for path in selection:
                 data.append(str(path[0]))
-            elif target == 'MODEL_ROW_EXTERN':
-                it = model.get_iter(path)
-                nfile, npage, angle = model.get(it, 2, 3, 6)
-                crop = model.get(it, 7, 8, 9, 10)
-                pdfdoc = self.pdfqueue[nfile - 1]
-                data.append('\n'.join([pdfdoc.filename,
-                                       str(npage),
-                                       str(angle)] +
-                                      [str(side) for side in crop]))
-        if data:
-            data = '\n;\n'.join(data)
-            selection_data.set(selection_data.get_target(), 8, data.encode())
+            if data:
+                data = '\n;\n'.join(data)
+        elif target == 'MODEL_ROW_EXTERN':
+            self.target_is_intern = False
+            data = self.copy_pages()
+        else:
+            return
+        selection_data.set(selection_data.get_target(), 8, data.encode())
 
     def iv_dnd_received_data(self, iconview, context, x, y,
                              selection_data, _target_id, etime):
@@ -783,62 +952,61 @@ class PdfArranger(Gtk.Application):
 
         model = iconview.get_model()
         data = selection_data.get_data()
-        if data:
-            data = data.decode().split('\n;\n')
-            item = iconview.get_dest_item_at_pos(x, y)
-            if item:
-                path, position = item
-                ref_to = Gtk.TreeRowReference.new(model, path)
-            else:
-                ref_to = None
-                position = Gtk.IconViewDropPosition.DROP_RIGHT
-                if len(model) > 0:  # find the iterator of the last row
-                    row = model[-1]
-                    ref_to = Gtk.TreeRowReference.new(model, row.path)
-            if ref_to:
-                before = (position == Gtk.IconViewDropPosition.DROP_LEFT
-                          or position == Gtk.IconViewDropPosition.DROP_ABOVE)
-                target = selection_data.get_target().name()
-                if target == 'MODEL_ROW_INTERN':
-                    move = context.get_actions() & Gdk.DragAction.MOVE
-                    self.undomanager.commit("Move" if move else "Copy")
-                    self.set_unsaved(True)
-                    data.sort(key=int, reverse=not before)
-                    ref_from_list = [Gtk.TreeRowReference.new(model, Gtk.TreePath(p))
-                                     for p in data]
-                    iter_to = self.model.get_iter(ref_to.get_path())
-                    for ref_from in ref_from_list:
-                        row = model[model.get_iter(ref_from.get_path())]
-                        if before:
-                            model.insert_before(iter_to, row[:])
-                        else:
-                            model.insert_after(iter_to, row[:])
-                    if move:
-                        for ref_from in ref_from_list:
-                            model.remove(model.get_iter(ref_from.get_path()))
+        if not data:
+            return
+        data = data.decode().split('\n;\n')
+        item = iconview.get_dest_item_at_pos(x, y)
+        if item:
+            path, position = item
+            ref_to = Gtk.TreeRowReference.new(model, path)
+        else:
+            ref_to = None
+            position = Gtk.IconViewDropPosition.DROP_RIGHT
+            if len(model) > 0:  # find the iterator of the last row
+                row = model[-1]
+                ref_to = Gtk.TreeRowReference.new(model, row.path)
+        before = (position == Gtk.IconViewDropPosition.DROP_LEFT
+                  or position == Gtk.IconViewDropPosition.DROP_ABOVE)
+        target = selection_data.get_target().name()
+        if target == 'MODEL_ROW_INTERN':
+            move = context.get_actions() & Gdk.DragAction.MOVE
+            self.undomanager.commit("Move" if move else "Copy")
+            self.set_unsaved(True)
+            data.sort(key=int, reverse=not before)
+            ref_from_list = [Gtk.TreeRowReference.new(model, Gtk.TreePath(p))
+                             for p in data]
+            iter_to = self.model.get_iter(ref_to.get_path())
+            for ref_from in ref_from_list:
+                row = model[model.get_iter(ref_from.get_path())]
+                if before:
+                    it = model.insert_before(iter_to, row[:])
+                else:
+                    it = model.insert_after(iter_to, row[:])
+                path = model.get_path(it)
+                iconview.select_path(path)
+            if move:
+                for ref_from in ref_from_list:
+                    model.remove(model.get_iter(ref_from.get_path()))
 
-                elif target == 'MODEL_ROW_EXTERN':
-                    pageadder = PageAdder(self)
-                    pageadder.move(ref_to, before)
-                    if not before:
-                        data.reverse()
-                    while data:
-                        tmp = data.pop(0).split('\n')
-                        filename = tmp[0]
-                        npage, angle = [int(k) for k in tmp[1:3]]
-                        crop = [float(side) for side in tmp[3:7]]
-                        pageadder.addpages(filename, npage, angle, crop)
-                    if pageadder.commit() and context.get_actions() & Gdk.DragAction.MOVE:
-                        context.finish(True, True, etime)
+        elif target == 'MODEL_ROW_EXTERN':
+            if not item and self.is_between_items(iconview, x, y):
+                context.finish(False, False, etime)
+                return
+            if not ref_to:
+                data.reverse()
+            changed = self.paste_pages(data, before, ref_to, select_added=True)
+            if changed and context.get_actions() & Gdk.DragAction.MOVE:
+                context.finish(True, True, etime)
 
     def iv_dnd_data_delete(self, _widget, _context):
         """Delete pages from a pdfarranger instance after they have
         been moved to another instance."""
-        selection = self.iconview.get_selected_items()
-        if len(selection) == 0:
+        if self.target_is_intern and os.name == 'nt':
+            # Workaround for windows
             # On Windows this method is triggered even for drag & drop within the same
             # pdfarranger instance
             return
+        selection = self.iconview.get_selected_items()
         self.undomanager.commit("Move")
         self.set_unsaved(True)
         model = self.iconview.get_model()
@@ -847,9 +1015,31 @@ class PdfArranger(Gtk.Application):
             path = ref_del.get_path()
             model.remove(model.get_iter(path))
 
-    def iv_dnd_motion(self, _iconview, _context, _x, y, _etime):
-        """Handles the drag-motion signal in order to auto-scroll the view"""
+    def is_between_items(self, iconview, x, y):
+        """Find out if drag location is between items."""
+        model = iconview.get_model()
+        if len(model) == 0:
+            return False
+        last_row = model[-1]
+        _x, _y, w, _h = self.cellthmb.do_get_size(iconview)
+        x_step = w
+        y_step = iconview.get_row_spacing() + 2 * iconview.get_item_padding()
+        xy_test = [(x - x_step, y),             # left
+                   (x + x_step, y),             # right
+                   (x, y + y_step),             # down
+                   (x - x_step, y + y_step),    # left-down
+                   (x + x_step, y + y_step)]    # right-down
 
+        for x_t, y_t in xy_test:
+            if x_t < 0:
+                x_t = 0
+            path = iconview.get_path_at_pos(x_t, y_t)
+            if path and not (path == last_row.path and x_t < x):
+                return True
+        return False
+
+    def iv_dnd_motion(self, iconview, _context, x, y, _etime):
+        """Handles auto-scroll when drag up/down. Also reject drop to location between items."""
         autoscroll_area = 40
         sw_vadj = self.sw.get_vadjustment()
         sw_height = self.sw.get_allocation().height
@@ -867,6 +1057,10 @@ class PdfArranger(Gtk.Application):
             GObject.source_remove(self.iv_auto_scroll_timer)
             self.iv_auto_scroll_timer = None
 
+        item = iconview.get_dest_item_at_pos(x, y)
+        if not item and self.is_between_items(iconview, x, y):
+            iconview.stop_emission('drag_motion')
+
     def iv_dnd_leave_end(self, _widget, _context, _ignored=None):
         """Ends the auto-scroll during DND"""
 
@@ -965,7 +1159,8 @@ class PdfArranger(Gtk.Application):
         ne = len(selection) > 0
         for a, e in [("reverse-order", self.reverse_order_available(selection)),
                      ("delete", ne), ("duplicate", ne), ("crop", ne), ("rotate", ne),
-                     ("export-selection", ne), ("split", ne)]:
+                     ("export-selection", ne), ("cut", ne), ("copy", ne),
+                     ("split", ne)]:
             self.window.lookup_action(a).set_enabled(e)
 
     def sw_dnd_received_data(self, _scrolledwindow, _context, _x, _y,
@@ -976,7 +1171,7 @@ class PdfArranger(Gtk.Application):
             for uri in selection_data.get_uris():
                 filename = get_file_path_from_dnd_dropped_uri(uri)
                 pageadder.addpages(filename)
-            pageadder.commit()
+            pageadder.commit(select_added=False, add_to_undomanager=True)
 
     def sw_button_press_event(self, _scrolledwindow, event):
         """Unselects all items in iconview on mouse click in scrolledwindow"""
@@ -1047,7 +1242,6 @@ class PdfArranger(Gtk.Application):
     def split_pages(self, _action, _parameter, _unknown):
         """ Split selected pages """
         model = self.iconview.get_model()
-        selection = self.iconview.get_selected_items()
         self.set_unsaved(True)
         self.undomanager.commit("Split")
         # selection is a list of 1-tuples, not in order
@@ -1210,7 +1404,7 @@ class PdfArranger(Gtk.Application):
         self.undomanager.commit("Reorder")
         model.reorder(new_order)
 
-    def about_dialog(self, _widget, _data=None):
+    def about_dialog(self, _action, _parameter, _unknown):
         about_dialog = Gtk.AboutDialog()
         about_dialog.set_transient_for(self.window)
         about_dialog.set_modal(True)
@@ -1224,12 +1418,15 @@ class PdfArranger(Gtk.Application):
         about_dialog.add_credit_section('Maintainers and contributors', [
             'https://github.com/jeromerobert/pdfarranger/graphs/contributors'])
         about_dialog.set_website_label(WEBSITE)
-        about_dialog.set_logo_icon_name(DOMAIN)
+        about_dialog.set_logo_icon_name(ICON_ID)
         about_dialog.set_license(LICENSE)
         about_dialog.connect('response', lambda w, *args: w.destroy())
         about_dialog.connect('delete_event', lambda w, *args: w.destroy())
         about_dialog.show_all()
 
+    def __update_num_pages(self, model, _path, _itr=None, _user_data=None):
+        self.uiXML.get_object("num_pages").set_text(str(len(model)))
+
     def error_message_dialog(self, msg, msg_type=Gtk.MessageType.ERROR):
         error_msg_dlg = Gtk.MessageDialog(flags=Gtk.DialogFlags.MODAL,
                                           type=msg_type, parent=self.window,
@@ -1286,7 +1483,13 @@ class PageAdder(object):
                 break
 
         if not pdfdoc:
-            pdfdoc = PDFDoc(filename, self.app.tmp_dir)
+            try:
+                pdfdoc = PDFDoc(filename, self.app.tmp_dir)
+            except GLib.Error as e:
+                message = e.message + ': ' + filename
+                print(message, file=sys.stderr)
+                self.app.error_message_dialog(message)
+                return
             self.app.import_directory = os.path.split(filename)[0]
             self.app.export_directory = self.app.import_directory
             self.app.pdfqueue.append(pdfdoc)
@@ -1313,22 +1516,28 @@ class PageAdder(object):
                                w, h,                 # 11-12
                                2.))                  # 13 FIXME
 
-    def commit(self):
+    def commit(self, select_added, add_to_undomanager):
         if len(self.pages) == 0:
             return False
-        self.app.undomanager.commit("Add")
-        self.app.set_unsaved(True)
+        if add_to_undomanager:
+            self.app.undomanager.commit("Add")
+            self.app.set_unsaved(True)
         for p in self.pages:
-            it = self.app.model.append(p)
             if self.treerowref:
                 iter_to = self.app.model.get_iter(self.treerowref.get_path())
                 if self.before:
-                    self.app.model.move_before(it, iter_to)
+                    it = self.app.model.insert_before(iter_to, p)
                 else:
-                    self.app.model.move_after(it, iter_to)
+                    it = self.app.model.insert_after(iter_to, p)
+            else:
+                it = self.app.model.append(p)
+            if select_added:
+                path = self.app.model.get_path(it)
+                self.app.iconview.select_path(path)
             self.app.update_geometry(it)
         GObject.idle_add(self.app.retitle)
         GObject.idle_add(self.app.render)
+        self.pages = []
         return True
 
 
